/**
 * test logger
 */


import {expect} from 'chai'
import {
  Logger,
  FileWriter,
} from '../fourdollar'
import * as $4 from '../fourdollar'
import {
  readFile,
  remove,
} from 'fs-extra'
import {
  join,
} from 'path'
import * as stop from 'stop.js'


describe('Logger', () => {
  describe('Logger', () => {
    const logger = new Logger('Hello')
    const origin = Logger.writer
    const oriFormat = Logger.format

    before(() => {
      Logger.writer = {
        log: null,
        error: null,
      }
    })

    after(() => {
      Logger.writer = origin
      Logger.format = oriFormat
    })

    it('Logger#log()', done => {
      Logger.writer.log = (msg) => {
        const reg = /\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}\.\d{3} > Hello - foo bar/
        expect(msg).to.match(reg)
        done()
      }
      logger.log('foo', 'bar')
    })

    it('Logger#error()', done => {
      Logger.writer.error = (msg) => {
        const reg = /\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}\.\d{3} > Hello - foo bar/
        expect(msg).to.match(reg)
        done()
      }
      logger.error('foo', 'bar')
    })

    it('Logger.format: default', done => {
      Logger.writer.log = (msg) => {
        expect(msg).to.match(/\d{4}-\d\d-\d\d \d\d:\d\d:\d\d.\d{3} > Hello - foobar/)
        done()
      }
      logger.log('foobar')
    })

    it('Logger.format = ":time: > [:name:] :msg:"', done => {
      Logger.format = ':time: > [:name:] :msg:'
      Logger.writer.log = (msg) => {
        expect(msg).to.match(/\d{4}-\d\d-\d\d \d\d:\d\d:\d\d.\d{3} > \[Hello\] foobar/)
        done()
      }
      logger.log('foobar')
    })
  })

  describe('FileWriter: normal stream', () => {
    const logger = new Logger('World')
    const dir = join(__dirname, 'log')
    const file = 'normal.log'
    
    before(() => {
      return remove(dir)
    })
    
    it('StreamLogger#log(): normal stream', async () => {
      Logger.writer.link = new FileWriter(join(dir, file))
      logger.log('foobar')
      await stop(500)
      let content = await readFile(join(dir, file))
      const reg = /log: \d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}\.\d{3} > World - foobar/
      expect(content).to.match(reg)
    })
    
    it('StreamLogger#error(): normal stream', async () => {
      logger.error('foobar')
      await stop(500)
      let content = await readFile(join(dir, file))
      const reg = /err: \d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}\.\d{3} > World - foobar/
      expect(content.toString()).to.match(reg)
    })
  })

  describe('FileWriter: rotating stream', () => {
    const logger = new Logger('World')
    const dir = join(__dirname, 'log')
    const file = 'rotating.log'
    
    it('StreamLogger#log(): rotating stream', async () => {
      Logger.writer = new FileWriter(join(dir, file), '1d')
      logger.log('foobar')
      await stop(500)
      let content = await readFile(join(dir, file))
      const reg = /log: \d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}\.\d{3} > World - foobar/
      expect(content).to.match(reg)
    })
    
    it('StreamLogger#error(): rotating stream', async () => {
      logger.error('foobar')
      await stop(500)
      let content = await readFile(join(dir, file))
      const reg = /err: \d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}\.\d{3} > World - foobar/
      expect(content.toString()).to.match(reg)
    })
  })

  describe('format', () => {
    const logger = new Logger('foobar')

    it('Logger.format default', () => {

    })
  })
})