"use strict";
/**
 * test promisify
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var $4 = require("../fourdollar");
var chai_1 = require("chai");
describe('promisify()', function () {
    var testObj = {
        name: 'bynaki',
        greet: function (err, say, callback) {
            callback.apply(this, [err, say + ", " + this.name]);
        },
        _greet: null,
    };
    it('testObj', function (done) {
        testObj.greet(new Error('its error!!'), 'hello', function (err, greeting) {
            chai_1.expect(err.message).to.be.equal('its error!!');
            chai_1.expect(greeting).to.be.equal('hello, bynaki');
            done();
        });
    });
    it('기본인수 전달', function () {
        testObj._greet = $4.promisify(testObj.greet);
        return testObj._greet(undefined, 'hello')
            .then(function (greeting) {
            chai_1.expect(greeting).to.be.equal('hello, bynaki');
        });
    });
    it('에러 처리', function () {
        return testObj._greet(new Error('its error!!'), 'hello')
            .catch(function (err) {
            chai_1.expect(err.message).to.be.equal('its error!!');
        });
    });
    it('context를 지정하지 않았을 때 greet 메서드가 this를 참조하므로 에러가 나야한다.', function () {
        var greet = $4.promisify(testObj.greet);
        return greet(undefined, 'hello')
            .catch(function (err) {
            chai_1.expect(err).to.be.an('error');
        });
    });
    it('에러를 반환하지 않는 콜백이라면', function () {
        testObj._greet = $4.promisify(testObj.greet, false);
        return testObj._greet('not error!!', 'hello')
            .then(function (args) {
            chai_1.expect(args).to.be.length(2);
            chai_1.expect(args[0]).to.be.equal('not error!!');
            chai_1.expect(args[1]).to.be.equal('hello, bynaki');
        });
    });
    it('context를 직접 지정한다면', function () {
        var obj = {
            name: 'rabbit'
        };
        var greet = $4.promisify(testObj.greet, obj);
        return greet(undefined, 'hello')
            .then(function (greeting) {
            chai_1.expect(greeting).to.be.equal('hello, rabbit');
        });
    });
    it('hasErr와 context 둘다 지정', function () {
        var greet = $4.promisify(testObj.greet, false, testObj);
        return greet('not error!!', 'hello')
            .then(function (args) {
            chai_1.expect(args).to.be.length(2);
            chai_1.expect(args[0]).to.be.equal('not error!!');
            chai_1.expect(args[1]).to.be.equal('hello, bynaki');
        });
    });
    it('hasErr와 context를 순서를 바꿔 지정', function () {
        var greet = $4.promisify(testObj.greet, testObj, true);
        return greet(undefined, 'hello')
            .then(function (greeting) {
            chai_1.expect(greeting).to.be.equal('hello, bynaki');
        });
    });
    it('엄한 인수 전달', function (done) {
        try {
            var greet = $4.promisify(testObj.greet, testObj, 33);
        }
        catch (err) {
            done();
        }
    });
    it('async-await', function () {
        var greet = $4.promisify(testObj.greet, testObj);
        return (function () { return __awaiter(_this, void 0, void 0, function () {
            var greeting;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, greet(undefined, 'hello')];
                    case 1:
                        greeting = _a.sent();
                        chai_1.expect(greeting).to.be.equal('hello, bynaki');
                        return [2 /*return*/];
                }
            });
        }); })();
    });
});

//# sourceMappingURL=promisify.spec.js.map
